/* Copyright (C) 2022-2023 GSI Helmholtzzentrum fuer Schwerionenforschung, Darmstadt
   SPDX-License-Identifier: GPL-3.0-only
   Authors: Sergey Gorbunov, Sergei Zharko [committer] */

#include "CaUtils.h"

/// Namespace contains compile-time constants definition for the CA tracking algorithm
///
namespace cbm::algo::ca::utils
{
  /// \brief Stingstream output operation for simd data
  template<typename DataT>
  void PrintSIMDmsg(std::stringstream& msg, DataT& v)
  {
    if constexpr (std::is_same_v<DataT, float> or std::is_same_v<DataT, double>) {
      msg << v;
    }
    else {
      msg << "[";
      for (size_t i = 0; i < DataT::size() - 1; i++)
        msg << v[i] << ",";
      msg << v[DataT::size() - 1] << "]";
    }
  }

  /// \brief Checks, if a SIMD vector horizontally equal
  /// TODO: Find this method in the VC!
  template<>
  void CheckSimdVectorEquality(fvec v, const char* name)
  {
    bool ok = true;
    for (size_t i = 1; i < fvec::size(); i++) {
      ok = ok && (v[i] == v[0]);
    }
    if (!ok) {
      std::stringstream msg;
      msg << name << " SIMD vector is inconsistent, not all of the words are equal each other: ";
      PrintSIMDmsg(msg, v);
      throw std::logic_error(msg.str());
    }
  }
}  // namespace cbm::algo::ca::utils
