/* Copyright (C) 2024 FIAS Frankfurt Institute for Advanced Studies, Frankfurt / Main
   SPDX-License-Identifier: GPL-3.0-only
   Authors: Felix Weiglhofer [committer], Dominik Smith */

#include "Unpack.h"

#include "log.hpp"

using namespace cbm::algo::much;
using fles::Subsystem;

Unpack::Unpack(const ReadoutConfig& readout) : fReadout(readout)
{
  constexpr i64 SystemTimeOffset = -980;

  // Create one algorithm per component for Bmon and configure it with parameters
  auto equipIdsMuch = fReadout.GetEquipmentIds();
  for (auto& equip : equipIdsMuch) {
    std::unique_ptr<much::UnpackPar> par(new much::UnpackPar());
    const size_t numElinks = fReadout.GetNumElinks(equip);
    for (size_t elink = 0; elink < numElinks; elink++) {
      much::UnpackElinkPar elinkPar;
      elinkPar.fAddress    = fReadout.Map(equip, elink);  // Vector of MUCH addresses for this elink
      elinkPar.fTimeOffset = SystemTimeOffset;
      elinkPar.fChanMask   = fReadout.MaskMap(equip, elink);
      par->fElinkParams.push_back(elinkPar);
    }
    fAlgos[equip].SetParams(std::move(par));
    L_(debug) << "--- Configured equipment " << equip << " with " << numElinks << " elinks";
  }
  L_(info) << "--- Configured " << fAlgos.size() << " unpacker algorithms for MUCH.";
}

Unpack::Result_t Unpack::operator()(const fles::Timeslice& ts) const
{
  constexpr int SystemVersion = 0x20;
  return DoUnpack(Subsystem::MUCH, ts, SystemVersion);
}
