/* Copyright (C) 2023 Facility for Antiproton and Ion Research in Europe, Darmstadt
   SPDX-License-Identifier: GPL-3.0-only
   Authors: Pierre-Alain Loizeau [committer] */

#ifndef CBM_SERVICES_HISTSERV_TESTER_APPLICATION_H
#define CBM_SERVICES_HISTSERV_TESTER_APPLICATION_H 1

#include <zmq.hpp>

#include "ProgramOptions.h"

namespace cbm::services::histserv_tester
{

  class Application {


  public:
    /** @brief Standard constructor, initialises the application
     ** @param opt  **/
    explicit Application(ProgramOptions const& opt);

    /** @brief Copy constructor forbidden **/
    Application(const Application&) = delete;

    /** @brief Assignment operator forbidden **/
    void operator=(const Application&) = delete;

    /** @brief Destructor **/
    ~Application() = default;

    /** @brief Run the application **/
    void Exec();

  private:
    template<class Object>
    void PrepareAndSendMsg(Object& obj, zmq::send_flags flags);

  private:
    ProgramOptions const& fOpt;  ///< Program options object

    /// Interface
    zmq::context_t fZmqContext {1};
    zmq::socket_t fZmqSocket {fZmqContext, ZMQ_PUSH};
  };

}  // namespace cbm::services::histserv_tester

#endif /* CBM_SERVICES_HISTSERV_TESTER_APPLICATION_H */
